/* -*- c -*-
 * Time-stamp: <2005-04-17 10:06:14 rsmith>
 * 
 * lamprop/latex_out.c
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>     /* for assert() */
#include <math.h>       /* for M_PI */
#include <stdio.h>      /* for printf() */
#include "macros.h"
#include "lamprop.h"

#ifndef PACKAGE
#error You must define PACKAGE as the name of the application!
#endif

#ifndef VERSION
#error You must define VERSION as the version of the application!
#endif

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: latex_out.c 36 2005-04-17 09:40:02Z rsmith $";
#endif

static void latex_engprop(const struct laminate *pl)
{
        int t = 1;
        struct lamina *l;

	puts("  \\parbox{\\textwidth}{\\centering");
	puts("    \\begin{tabular}[t]{rcrl}");
	puts("      \\multicolumn{4}{c}{\\small\\textbf{Laminate "\
	       "stacking}}\\\\[0.1em]");
	puts("      \\toprule %% \\usepackage{booktabs}");
	puts("      Layer & Weight & Angle & Fiber type\\\\");
	puts("            & [gr/m$^2$] & [$\\circ$] & \\\\");
	puts("      \\midrule");
	l = (struct lamina *)list_head(pl->lamina);
        while (l != NULL) {
                printf("      %d & %4.0f & %5.0f & %s\\\\\n", 
		       t++, l->weight, l->angle*180/M_PI,l->f->name);
                l = (struct lamina *)list_next(LISTP(l));
        }
	puts("      \\bottomrule");
	puts("    \\end{tabular}\\hspace{0.02\\textwidth}");
	puts("    \\begin{tabular}[t]{rrl}");
	puts("      \\multicolumn{3}{c}{\\small\\textbf{Engineering"\
	       " properties}}\\\\[0.1em]");
	puts("      \\toprule");
	puts("      Property & Value & Dimension\\\\");
	puts("      \\midrule");
	printf("      $v_f$ & %4.2f &-\\\\\n", pl->vf);
	printf("      $w_f$ & %4.2f &-\\\\\n", pl->wf);
	printf("      thickness & %.3g & mm\\\\\n", pl->t);
	printf("      density & %.3g & gr/cm$^3$\\\\\n", pl->rho);
	printf("      weight & %.0f & gr/m$^2$\\\\\n", 
	       pl->rho*pl->t/10*100*100 );
	printf("      resin & %.0f & gr/m$^2$\\\\\n", pl->rc);
	puts("      \\midrule");
	printf("      $E_x$ & %8.0f & MPa\\\\\n", pl->Ex);
	printf("      $E_y$ & %8.0f & MPa\\\\\n", pl->Ey);
	printf("      $G_{xy}$ & %8.0f & MPa\\\\\n", pl->Gxy);
	printf("      $\\nu_{xy}$ & %g &-\\\\\n", pl->Vxy);
	printf("      $\\nu_{yx}$ & %g &-\\\\\n", pl->Vyx);
	printf("      $\\alpha_x$ & %g & K$^{-1}$\\\\\n", pl->cte_x);
	printf("      $\\alpha_y$ & %g & K$^{-1}$\\\\\n", pl->cte_y);
	puts("      \\bottomrule");
	puts("    \\end{tabular}");
	puts("  }\\vspace{5mm}");
}

static void latex_mat(const struct laminate *pl)
{
        int t = 0;

	puts("  \\vbox{");
	puts("    \\vbox{\\small\\textbf{Stiffness matrix}\\\\");
	puts("      \\tiny\\[\\left\\{\\begin{array}{c}");
	puts("          N_x\\\\ N_y\\\\ N_{xy}\\\\ M_x\\\\ M_y\\\\"\
	       " M_{xy}");
	puts("        \\end{array}\\right\\} = ");
	puts("      \\left|\\begin{array}{cccccc}");
        for (t=0; t<6; t++)
                printf("          %6.0f & %6.0f & %6.0f & %6.0f & %6.0f"\
		       " & %6.0f\\\\\n", pl->ABD[t][0], pl->ABD[t][1], 
                       pl->ABD[t][2], pl->ABD[t][3],  
                       pl->ABD[t][4], pl->ABD[t][5]); 
	puts("          \\end{array}\\right| \\times");
	puts("        \\left\\{\\begin{array}{c}");
	puts("            \\epsilon^0_x\\\\[3pt] \\epsilon^0_y\\\\[3pt] "\
	       "\\gamma^0_{xy}\\\\[3pt]");
	puts("            \\kappa^0_x\\\\[3pt] \\kappa^0_y\\\\[3pt] "\
	       "\\kappa^0_{xy}");
	puts("          \\end{array}\\right\\}\\]");
	puts("    }");
	puts("    \\vbox{\\small\\textbf{Compliance matrix}\\\\");
	puts("      \\tiny\\[\\left\\{\\begin{array}{c}");
	puts("            \\epsilon^0_x\\\\[3pt] \\epsilon^0_y\\\\[3pt] "\
	       "\\gamma^0_{xy}\\\\[3pt]");
	puts("            \\kappa^0_x\\\\[3pt] \\kappa^0_y\\\\[3pt] "\
	     "\\kappa^0_{xy}");
	puts("          \\end{array}\\right\\} = \\left|\\begin{array}"\
	       "{cccccc}");
        for (t=0; t<6; t++)
                printf("          %6.3g & %6.3g & %6.3g & %6.3g & %6.3g"\
		       " & %6.3g\\\\\n", pl->abd[t][0]*1e6, pl->abd[t][1]*1e6, 
		       pl->abd[t][2]*1e6, pl->abd[t][3]*1e6,
		       pl->abd[t][4]*1e6, pl->abd[t][5]*1e6); 
	puts("          \\end{array}\\right|\\times10^{-6}\\times");
	puts("        \\left\\{\\begin{array}{c}");
	puts("            N_x\\\\ N_y\\\\ N_{xy}\\\\ M_x\\\\ M_y\\\\ "\
	       "M_{xy}");
	puts("          \\end{array}\\right\\}\\]\\\\");
	puts("    }");
	puts("  }");	
}

void latex_output(const struct laminate *pl, unsigned flags)
{

	assert(pl != NULL);

	puts("\\begin{table}[htbp]");
	puts("  \\renewcommand{\\arraystretch}{1.2}");
	printf("  \\caption{\\label{tab:%s}properties of %s}\n", 
	       pl->name, pl->name);
	puts("  \\centering\\footnotesize{\\rule{0pt}{10pt}");
	puts("  \\tiny calculated by "PACKAGE" "VERSION"\\\\[3pt]}\n");
	if ((flags & OF_ENGPROP) == OF_ENGPROP)
		latex_engprop(pl);
	if ((flags & OF_MAT) == OF_MAT)
		latex_mat(pl);
	puts("\\end{table}\n");
}
/* EOF latex_out.c */
