/* -*- c -*-
 * Time-stamp: <2005-04-14 23:24:12 rsmith>
 * 
 * list/list.h
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#ifndef LIST_H
#define LIST_H

#ifdef __cplusplus
#  define BEGIN_C_DECLS extern "C" {
#  define END_C_DECLS   }
#else                           /* !__cplusplus */
#  define BEGIN_C_DECLS
#  define END_C_DECLS
#endif                          /* __cplusplus */

#ifndef NULL
#define NULL (void*)0
#endif

/* Convert a pointer to a list pointer. */
#define LISTP(p) ((struct list *)(p))

BEGIN_C_DECLS

/* Structure for making linked lists. */
struct list 
{
        struct list *prev;
        struct list *next;
};

/* list.c */
/* Get the next node in a list, or NULL if not available. */
extern struct list *list_next(struct list *l);
/* Get the previous node in a list, or NULL if not available. */
extern struct list *list_prev(struct list *l);
/* Get the node at the head of the list, or NULL if not available. */
extern struct list *list_head(struct list *l);
/* Get the node at the end of the list, or NULL if not available. */
extern struct list *list_tail(struct list *l);
/* Get the n-th node in the list, starting from 0. */
extern struct list *list_nth(struct list *l, int n);
/* Add a node at the end of the list, returns the new list. */
extern struct list *list_append(struct list *l, struct list *newitem);
/* Add a node at the beginning of the list, returns the new list. */
extern struct list *list_prepend(struct list *l, struct list *newitem);

END_C_DECLS
#endif				/* LIST_H */
/* EOF list.h */
