/* -*- c -*-
 * Time-stamp: <2005-04-17 10:07:42 rsmith>
 * 
 * lamprop/text_out.c
 * Copyright (C) 2002--2005 R.F. Smith <rsmith@xs4all.nl>. 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* If you don't want debugging, #define NDEBUG */

#include <assert.h>	/* for assert() */
#include <math.h>       /* for M_PI */
#include <stdio.h>	/* for printf() */
#include "macros.h"
#include "lamprop.h"

#ifndef PACKAGE
#error You must define PACKAGE as the name of the application!
#endif

#ifndef VERSION
#error You must define VERSION as the version of the application!
#endif

/* This enables you to see which files have been used in building a
 * binary. N.B, __attribute is a GCC extension, so we test for __GNUC__. */
#ifdef __GNUC__
static char *SCMID __attribute__ ((unused)) = 
"$Id: text_out.c 36 2005-04-17 09:40:02Z rsmith $";
#endif

static void text_engprop(const struct laminate *pl)
{
	int t = 1;
	struct lamina *l;

	printf("laminate: %s\n", pl->name);
	printf("thickness: %.3g mm, density %.3g gr/cm3\n", pl->t, pl->rho);
	printf("fiber volume fraction %g%%, fiber weight fraction %4.1f%%\n", 
	       pl->vf*100, pl->wf*100);
	printf("laminate weight: %.0f gr/m2, resin consumption: %.0f gr/m2\n", 
	       pl->rho*pl->t/10*100*100, pl->rc);
	puts("num weight angle fiber");
	l = (struct lamina *)list_head(pl->lamina);
	while (l != NULL) {
		printf("%3d %4.0f %5.0f   %s\n", t++, l->weight, 
		       l->angle*180/M_PI,l->f->name);
		l = (struct lamina *)list_next(LISTP(l));
	}
	printf("E_x  = %g MPa\n", pl->Ex);
	printf("E_y  = %g MPa\n", pl->Ey);
	printf("G_xy = %g MPa\n", pl->Gxy);
	printf("v_xy = %g\n", pl->Vxy);
	printf("v_yx = %g\n", pl->Vyx);
	printf("cte_x = %g cte_y = %g\n", pl->cte_x, pl->cte_y);
}

static void text_mat(const struct laminate *pl)
{
	int t = 0;

	puts("ABD matrix:");
	for (t=0; t<6; t++)
		printf("| %6.0f %6.0f %6.0f %6.0f %6.0f %6.0f |\n",
		       pl->ABD[t][0], pl->ABD[t][1], 
		       pl->ABD[t][2], pl->ABD[t][3],  
		       pl->ABD[t][4], pl->ABD[t][5]); 
	puts("abd matrix:");
	for (t=0; t<6; t++)
		printf("| %10g %10g %10g %10g %10g %10g |\n",
		       pl->abd[t][0], pl->abd[t][1], 
		       pl->abd[t][2], pl->abd[t][3],  
		       pl->abd[t][4], pl->abd[t][5]);
}


void text_output(const struct laminate *pl, unsigned flags)
{
	assert(pl!=NULL);
	puts("Generated by "PACKAGE" "VERSION);
	if ((flags & OF_ENGPROP) == OF_ENGPROP)
		text_engprop(pl);
	if ((flags & OF_MAT) == OF_MAT)
		text_mat(pl);
	puts("");
}
/* EOF text_out.c */
